
from ecsm_tool.ecsm_cli.cli import CLI
from ecsm_tool.ecsm_core.repo import ECSM_REPO
from ecsm_tool.ecsm_cli.utils import *
import json

class CLI_IMAGES(CLI):
    """镜像管理命令"""

    def __init__(self) -> None:
        super().__init__("images", "镜像管理")

        self.add_subcommand(
            name="upload",
            help="镜像上传",
            func=self.upload,
            args=[
                {"flags": ["-i", "--ip"], "options": {"help": "ECSM 服务器 IP 地址", "type": str, "required": True}},
                {"flags": ["-p", "--port"], "options": {"help": "ECSM 服务器端口", "type": str, "required": True}},
                {"flags": ["-f", "--file"], "options": {"help": "目标文件路径", "type": str, "required": True}},
            ],
        )

        self.add_subcommand(
            name="download",
            help="镜像下载",
            func=None,
            args=[
                {"flags": ["-i", "--ip"], "options": {"help": "ECSM 服务器 IP 地址", "type": str, "required": True}},
                {"flags": ["-p", "--port"], "options": {"help": "ECSM 服务器端口", "type": str, "required": True}},
                {"flags": ["-f", "--file"], "options": {"help": "目标文件路径", "type": str, "required": True}},
                {"flags": ["--id"], "options": {"help": "目标镜像 ID", "type": str, "required": True}},
            ],
        )

        self.add_subcommand(
            name="list",
            help="查询镜像列表",
            func=self.list,
            args=[
                {"flags": ["-i", "--ip"], "options": {"help": "ECSM 服务器 IP 地址", "type": str, "required": True}},
                {"flags": ["-p", "--port"], "options": {"help": "ECSM 服务器端口", "type": str, "required": True}},
                {"flags": ["-d", "--delete"], "options": {"help": "需要删除的仓库 ID", "type": str, "required": False}},
                {"flags": ["--id"], "options": {"help": "目标镜像 ID", "type": str, "required": False}},
            ],
        )

        self.add_subcommand(
            name="summary",
            help="查询镜像数量",
            func=self.summary,
            args=[
                {"flags": ["-i", "--ip"], "options": {"help": "ECSM 服务器 IP 地址", "type": str, "required": True}},
                {"flags": ["-p", "--port"], "options": {"help": "ECSM 服务器端口", "type": str, "required": True}},
            ],
        )

    @staticmethod
    def upload(args) -> None:
        if not ecsm_host_valid(args):
            return

        ecsm = ECSM_REPO(args.ip, args.port)
        code, msg  = ecsm.image_upload(args.file, "")
        if code == 0:
            print(f"文件上传成功")
            return

        print(f"上传失败 {msg}")

    @staticmethod
    def download(args) -> None:
        if not ecsm_host_valid(args):
            return

        save_file = getattr(args, "file")
        image_id  = getattr(args, "id")

        if not save_file or not image_id:
            print("请输入有效的目标文件路径和 ECSM 镜像 ID")
            return

        ecsm = ECSM_REPO(args.ip, args.port)
        _, msg  = ecsm.image_download(save_file, image_id)
        print(msg)

    @staticmethod
    def list(args) -> None:
        if not ecsm_host_valid(args):
            return

        ecsm = ECSM_REPO(args.ip, args.port)
        if getattr(args, 'delete'):
            _, msg = ecsm.image_delete([getattr(args, 'delete')])
            print(msg)
            return

        if getattr(args, "id"):
            _, msg = ecsm.image_info(getattr(args, "id"))
            print(json.dumps(msg, indent=4, ensure_ascii=False))
            return

        code, img_list = ecsm.image_list()
        if code == 0:
            print("ECSM 镜像信息：")
            if not len(img_list):
                print("ECSM 仓库不存在任何镜像")
                return

            for img in img_list:
                print(f"""{img['name']} {img['id']}""")
        else:
            print("ECSM 镜像列表获取失败")

    @staticmethod
    def summary(args) -> None:
        if not ecsm_host_valid(args):
            return

        ecsm = ECSM_REPO(args.ip, args.port)
        code, data = ecsm.summary()
        if code == 0:
            fmt_out = f"""
    ECSM 仓库镜像数量信息：
        本地镜像：{data["local"]}
        远程镜像: {data["remote"]}
    """
            print(fmt_out)
        else:
            print("无法获取有效镜像信息")
