
import os
import hashlib
import requests
from tqdm import tqdm

class ECSM_REPO:
    """
    ECSM 镜像管理 - 镜像仓库
    """

    def __init__(self, srv_ip, srv_port):
        self.srv_ip = srv_ip
        self.srv_port = srv_port

    def __upload_chunk(self, file: dict, splits: int, index: int, data_size: int) -> tuple[int, str]:
        offset = (index - 1) *data_size

        with open(file["path"], "rb") as f:
            f.seek(offset)
            data = f.read(data_size)

        # 请求参数
        # description   string  否  文件描述信息
        # total         int     是  镜像文件分片个数
        # index         int     是  当前上传片段索引，从 1 开始
        # imageHash     string  是  完整的镜像文件 hash。注意：非当前镜像片段 hash
        # totalSize     int     是  完整镜像文件总大小，单位为 Byte
        # bufferSize    int     是  当前镜像片段大小，单位为 Byte
        # offset        int     是  切片大小，单位为 Byte

        headers = {
            "description": file["desc"],
            "imageHash":   file["hash"],
            "totalSize":   str(file["size"]),
            "total":       str(splits),
            "index":       str(index),
            "bufferSize":  str(len(data)),
            "offset":      str(data_size),
            "Content-Type": "application/octet-stream",
        }

        # 发送 POST 请求上传文件片段
        url = f"http://{self.srv_ip}:{self.srv_port}/api/v1/image"

        response = requests.post(url, headers=headers, data=data)
        if response.status_code == 200:
            self.progress.update(len(data))
            msg = f"Uploaded chunk {index} successfully."
        else:
            msg = f"Error: Failed to upload chunk {index}. Status code: {response.status_code}"

        return 0 if response.status_code == 200 else -1, msg

    def __list_page(self, isLocal: bool, page_num: int, page_size: int) -> tuple[int, list]:

        repo_type = "local" if isLocal > 0 else "remote"

        url = f"http://{self.srv_ip}:{self.srv_port}/api/v1/image?"
        url += f"registryId={repo_type}&pageNum={page_num}&pageSize={page_size}"

        response = requests.get(url)
        if response.status_code == 200:
            imgs_raw = response.json().get("data", {}).get("list", [])
            imgs = [
                {
                        "name": img.get('name', ''),
                        "arch": img.get('arch', ''),
                        "os":   img.get('os', ''),
                        "desc": img.get('description', ''),
                        "tag":  img.get('tag', ''),
                        "id":   img.get('id', '')
                }
                for img in imgs_raw
            ]
            return 0, imgs

        return -1, []

    def image_upload(self, file_path: str, file_desc: str) -> tuple[int, str]:
        """
        ECSM 镜像管理 - 镜像仓库 - 上传镜像
        """

        if not os.path.exists(file_path):
            return -1, f"文件不存在： {file_path} "

        file_info = os.stat(file_path)
        file_size = file_info.st_size

        # 计算文件的哈希值
        file_hash_md5 = hashlib.md5()
        with open(file_path, 'rb') as f:
            for chunk in iter(lambda: f.read(4096), b""):
                file_hash_md5.update(chunk)

        target_file = {
            "desc": file_desc,
            "path": file_path,
            "size": file_size,
            "hash": file_hash_md5.hexdigest()
        }

        taget_splits_size = 1 * 1024 * 1024
        target_splits = file_size // taget_splits_size + 1 if file_size % taget_splits_size else 0

        self.progress = tqdm(total=file_size, unit="B", unit_scale=True, desc="上传进度")
        for index in range(0, target_splits):
            code, msg = self.__upload_chunk(target_file, target_splits, index + 1, taget_splits_size)
            if code < 0:
                self.progress.close()
                return -1, msg

        self.progress.close()
        return 0, "上传成功"

    def image_upload_progress(self, uploadId: str):
        """
        ECSM 镜像管理 - 镜像仓库 - 查询镜像上传进度
        """

        url = f"http://{self.srv_ip}:{self.srv_port}/api/v1/image/upload/{uploadId}"

        # 发送 POST 请求上传文件片段
        response = requests.get(url)
        if response.status_code == 200:
            return 0, response.content

        return -1, ""

    def image_list(self) -> tuple[int, list]:
        is_local = True
        page_num = 1
        page_size = 100

        code, imgs = self.__list_page(is_local, page_num, page_size)
        if code == 0:
            return code, imgs

        return -1, []

    def image_delete(self, id_list: list[str]) -> tuple[int, str]:
        if not id:
            return -1, "ID 无效"

        url = f"http://{self.srv_ip}:{self.srv_port}/api/v1/image"
        headers = {
            'Content-Type': 'application/json'
        }
        data = {
            "ids": [item for item in id_list]
        }

        response = requests.delete(url, headers=headers, json=data)
        if response.status_code != 200:
            return -1, "仓库删除 URL 请求失败"

        op_code = response.json().get('status')
        if int(op_code) != 200:
            return -1, "仓库删除失败"

        return 0, "仓库删除成功"

    def image_download(self, save_path: str, image_id: str) -> tuple[int, str]:
        if os.path.exists(save_path):
            return -1, f"文件已存在，保存路径: {save_path}"

        image_size = self.image_size(image_id)

        url = f"http://{self.srv_ip}:{self.srv_port}/api/v1/image/download?imageID={image_id}"

        try:
            response = requests.get(url, stream=True)
            print(response.headers)
            if response.status_code == 200:
                with open(save_path, "wb") as file:
                    for chunk in response.iter_content(chunk_size=1024):
                        if chunk:
                            file.write(chunk)
                return 0, f"镜像下载成功，保存路径: {save_path}"

            return -1, f"下载失败， HTTP 状态码: {response.status_code}"

        except requests.exceptions.RequestException as e:
            return -1, f"下载过程中出现错误: {e}"

    def image_info(self, image_id: str) -> tuple[int, dict]:
        if not image_id:
            return -1, {}

        url = f"http://{self.srv_ip}:{self.srv_port}/api/v1/registry/local/image/{image_id}"
        try:
            response = requests.get(url)
            if response.status_code == 200:
                return 0, response.json().get("data")
        except requests.exceptions.RequestException as e:
            return -1, {}

        return -1, {}

    def image_size(self, image_id: str) -> int:
        code, img_info = self.image_info(image_id)
        if code < 0:
            return -1

        try:
            size = int(img_info.get("size", 0))
        except (ValueError, TypeError):
            return 0

        return size if size > 0 else 0

    def summary(self) -> tuple[int, dict]:
        url = f"http://{self.srv_ip}:{self.srv_port}/api/v1/image/summary"

        response = requests.get(url)
        if response.status_code != 200:
            return -1, {}

        resp_body = response.json()
        resp_con = {
            "local": resp_body.get("data", {}).get("local"),
            "remote": resp_body.get("data", {}).get("remote")
        }

        return 0, resp_con
